/*
 * crackme/crackme.c --
 */

#define _POSIX_C_SOURCE 200809L

#include <stdio.h>
#include <string.h>
#include "crackme.h"

struct level {
    int (*is_good)(const char *);
    char *name;
};

static struct level levels [] = {
    { c00, "c00" },
    { c01, "c01" },
    { c02, "c02" },
    { c03, "c03" },
    { c04, "c04" },
    { NULL, NULL }
};

static int crack(struct level *l)
{
    char *line = NULL;
    size_t linecap = 0;
    ssize_t linelen;

    printf("(%s) password: ", l->name);
    fflush(stdout);
    linelen = getline(&line, &linecap, stdin);
    if (linelen < 0) {
        return -1;
    }
    if (linelen > 0 && line[linelen-1] == '\n') {
        line[linelen-1] = 0;
    }
    if (l->is_good(line)) {
        printf("(%s) success\n", l->name);
        return 0;            
    } else {
        printf("(%s) invalid\n", l->name);
        return 1;
    }
}

static struct level* lookup(const char *name)
{
    struct level *l;
    
    for (l = levels; l->name && strcmp(l->name, name) != 0; l++) ;
    return l->name ? l : NULL;
}

int main(int argc, char *argv[])
{
    struct level *l;
    int status = 0;

    if (argc == 1) {
        printf("usage: crackme level [level ...]\n\n");
        printf("       levels: ");
        for (l = levels; l->name; l++) {
            printf("%s ", l->name);
        }
        printf("\n");
        return 0;
    }

    for (int i = 1; i < argc; i++) {
        l = lookup(argv[i]);
        if (! l) {
            fprintf(stderr, "crackme: unknown level '%s'\n", argv[i]);
            status = 1;
            continue;
        }
        if (crack(l) < 0) {
            fprintf(stderr, "crackme: failed to read password\n");
            status = 1;
            break;
        }
    }

    return status;
}
